@@ -27,15 +27,17 @@
      Fixes:
 
      Add -r flag: Keith Owens <kaos@ocs.com.au> October 1999.
- 
+
      Rationalize common code for 32/64 bit architectures.
      Fix error message for modules.dep.
        Keith Owens <kaos@ocs.com.au> December 1999
      Add arch64().
        Keith Owens <kaos@ocs.com.au> December 1999.
+     Ignore any genksyms suffix when using -F.
+       Keith Owens <kaos@ocs.com.au> April 2000.
    */
 
-#ident "$Id: depmod.c 1.9 Wed, 15 Mar 2000 21:14:05 +1100 keith $"
+#ident "$Id: depmod.c 1.13 Thu, 20 Apr 2000 14:31:49 +1000 kaos $"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -49,6 +51,7 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <syslog.h>
+#include <limits.h>
 #include <elf.h>
 #include ELF_MACHINE_H
 
@@ -67,6 +70,8 @@ typedef struct SYMBOL{
 	struct MODULE *module;	/* Module declaring this symbol */
 	const char *name;
 	unsigned short hashval;
+	unsigned short hashval_nosuffix;
+	unsigned short length_nosuffix;
 	char status;
 } SYMBOL;
 
@@ -115,12 +120,15 @@ extern int quick;	/* Option -A */
  *	Add defined symbol to the head of the list of defined symbols.
  *	Return the new symbol.
  */
-static SYMBOL *addsym(const char *name, MODULE *module, SYM_STATUS status)
+static SYMBOL *addsym(const char *name, MODULE *module, SYM_STATUS status,
+		      int ignore_suffix)
 {
 	SYMBOL added;
 	SYMBOL *me;
 	unsigned short hashval = 0;
-	const char *pt = name;
+	const char *pt = name, *pt_R;
+	char *p;
+	int l;
 
 	for (pt = name; *pt; ++pt)
 		hashval = (hashval << 1) ^ *pt;
@@ -132,6 +140,26 @@ static SYMBOL *addsym(const char *name, MODULE *module, SYM_STATUS status)
 	added.hashval = hashval;
 	added.status = status;
 
+	/* Compute the hash without the trailing genksym suffix if required */
+	if (ignore_suffix && (pt_R = strstr(name, "_R"))) {
+		l = strlen(pt_R);
+		if (l >= 10) {	/* May be _R.*xxxxxxxx */
+			(void)strtoul(pt_R+l-8, &p, 16);
+			if (!*p) { /* It is _R.*xxxxxxxx */
+				added.length_nosuffix = pt_R - name;
+				hashval = 0;
+				for (pt = name; pt < pt_R; ++pt)
+					hashval = (hashval << 1) ^ *pt;
+				hashval %= 2048;
+				added.hashval_nosuffix = hashval;
+			}
+		}
+	}
+	else {
+		added.hashval_nosuffix = 0;
+		added.length_nosuffix = 0;
+	}
+
 	if (status == SYM_DEFINED) {
 		if (symavail == NULL ||
 		    symavail == maxsyms) {
@@ -146,8 +174,8 @@ static SYMBOL *addsym(const char *name, MODULE *module, SYM_STATUS status)
 		}
 		me = symavail++;
 		*me = added;
-		me->next = symhash[hashval];
-		symhash[hashval] = me;
+		me->next = symhash[added.hashval];
+		symhash[added.hashval] = me;
 	} else {
 		me = (SYMBOL *)xmalloc(sizeof(SYMBOL));
 		*me = added;
@@ -190,7 +218,11 @@ static void resolve(void)
 			   definition.  Symbol hash linked list has
 			   newest first, so we want the last match in
 			   the chain.  In particular, this prefers
-			   kernel symbols to that of any module.  */
+			   kernel symbols to that of any module.
+			   
+			   If we are ignoring the genksyms suffix in this
+			   comparision then there are two hashes to try.
+			 */
 
 			find = symhash[psym->hashval];
 			while (find) {
@@ -200,6 +232,19 @@ static void resolve(void)
 				}
 				find = find->next;
 			}
+			if (psym->status == SYM_UNDEF && psym->length_nosuffix) {
+				/* Try again ignoring the suffix */
+				find = symhash[psym->hashval_nosuffix];
+				while (find) {
+					if (strncmp(find->name, psym->name,
+					    psym->length_nosuffix) == 0 &&
+					    !find->name[psym->length_nosuffix]) {
+						psym->status = SYM_RESOLVED;
+						psym->module = find->module;
+					}
+					find = find->next;
+				}
+			}
 			if (psym->status == SYM_UNDEF)
 				resolved = 0;
 		}
@@ -211,7 +256,7 @@ static void resolve(void)
  *	Read the symbols in an object and register them in the symbol table.
  *	Return -1 if there is an error.
  */
-static int loadobj(const char *objname)
+static int loadobj(const char *objname, int ignore_suffix)
 {
 	static char *currdir;
 	static int currdir_len;
@@ -236,7 +281,7 @@ static int loadobj(const char *objname)
 	if ((fp = fopen(objname, "r")) == NULL)
 		return 1;
 
-	if (!(f = obj_load(fp))) {
+	if (!(f = obj_load(fp, ET_REL))) {
 		fclose(fp);
 		return -1;
 	}
@@ -282,7 +327,8 @@ static int loadobj(const char *objname)
 				if (ELFW(ST_BIND)(objsym->info) != STB_WEAK) {
 					undefs[n_undefs++] = addsym(objsym->name,
 								    mod,
-								    SYM_UNDEF);
+								    SYM_UNDEF,
+								    ignore_suffix);
 				}
 				continue;
 			}
@@ -295,7 +341,8 @@ static int loadobj(const char *objname)
 					/* Ignore leading "__ksymtab_" */
 					defsym[n_defsym++] = addsym(objsym->name + 10,
 								    mod,
-								    SYM_DEFINED);
+								    SYM_DEFINED,
+								    ignore_suffix);
 				}
 				continue;
 			}
@@ -311,7 +358,8 @@ static int loadobj(const char *objname)
 				if (ELFW(ST_BIND)(objsym->info) == STB_GLOBAL) {
 					defsym[n_defsym++] = addsym(objsym->name,
 								    mod,
-								    SYM_DEFINED);
+								    SYM_DEFINED,
+								    ignore_suffix);
 				}
 				continue;
 			}
@@ -328,7 +376,8 @@ static int loadobj(const char *objname)
 			     ) {
 				defsym[n_defsym++] = addsym(objsym->name,
 							    mod,
-							    SYM_DEFINED);
+							    SYM_DEFINED,
+							    ignore_suffix);
 			}
 		}
 	}
@@ -385,7 +434,7 @@ static int addksyms(char *file_syms)
 	int is_mapfile = 0;
 	int n_syms = 0;
 	int size;
-	char line[100];
+	char line[PATH_MAX];
 	char *p;
 
 	mod->name = "-";
@@ -396,8 +445,8 @@ static int addksyms(char *file_syms)
 
 	/* Fake the _mod_use_count_ symbol provided by insmod */
 	/* Dummy */
-	symtab[n_syms++] = addsym("mod_use_count_", mod, SYM_DEFINED);
-	symtab[n_syms++] = addsym("__this_module", mod, SYM_DEFINED);
+	symtab[n_syms++] = addsym("mod_use_count_", mod, SYM_DEFINED, 0);
+	symtab[n_syms++] = addsym("__this_module", mod, SYM_DEFINED, 0);
 
 	/*
 	 * Specification: depmod / kernel syms only
@@ -417,7 +466,7 @@ static int addksyms(char *file_syms)
 			error("Can't read %s", file_syms);
 			return -1;
 		}
-		if (!fgets(line, 100, fp)) {
+		if (!fgets(line, sizeof(line), fp)) {
 			fclose(fp);
 			error("premature EOF on %s", file_syms);
 			return -1;
@@ -442,9 +491,9 @@ static int addksyms(char *file_syms)
 		}
 
 		if (p)
-			symtab[n_syms++] = addsym(p, mod, SYM_DEFINED);
+			symtab[n_syms++] = addsym(p, mod, SYM_DEFINED, 0);
 
-		while (fgets(line, 100, fp)) {
+		while (fgets(line, sizeof(line), fp)) {
 			if (!is_mapfile && strchr(line, '['))
 				continue;
 			strtok(line, " \t\n"); /* Skip first word */
@@ -455,7 +504,7 @@ static int addksyms(char *file_syms)
 				p = strtok(NULL, " \t\n");
 			}
 			assert(n_syms < 10000);
-			symtab[n_syms++] = addsym(p, mod, SYM_DEFINED);
+			symtab[n_syms++] = addsym(p, mod, SYM_DEFINED, 0);
 		}
 		fclose(fp);
 	} else { /* Use the exported symbols from currently running kernel */
@@ -464,7 +513,7 @@ static int addksyms(char *file_syms)
 
 		for (ksym = ksyms; so_far < nksyms; ++so_far, ksym++) {
 			assert(n_syms < 10000);
-			symtab[n_syms++] = addsym((char *)ksym->name, mod, SYM_DEFINED);
+			symtab[n_syms++] = addsym((char *)ksym->name, mod, SYM_DEFINED, 0);
 		}
 	}
 
@@ -561,7 +610,7 @@ void depmod_usage(void)
 	"depmod -[aA] [-n -e -v -q -V]\n"
 	"      [-C configfile] [-F kernelsyms] [-b basedirectory] [forced_version]\n"
 	"depmod [-n -e -v -q] [-f kernelsyms] module1.o module2.o ...\n"
-	"If no arguments (expect options) are given, \"depmod -a\" is assumed\n"
+	"If no arguments (except options) are given, \"depmod -a\" is assumed\n"
 	"\n"
 	"depmod will output a dependancy list suitable for the modprobe utility.\n"
 	"depmod -a will find the list of modules to probe from the file\n"
@@ -573,18 +622,28 @@ void depmod_usage(void)
 	"\n"
 	"Normally depmod operates silently, reporting only the list of modules that\n"
 	"won't load properly (missing symbols).\n"
-	"Option -q will make depmod keep quiet about missing symbols\n"
-	"Option -e output all the unresolved symbol for a given module\n"
-	"Option -s output all error message to the syslog daemon\n"
-	"Option -v force a printout of all visited modules.\n"
-	"Option -n will write the dependency file on stdout only.\n"
-	"Option -V will show you the release version of depmod\n"
-	"Option -r will allow root to load modules that are not owned by root\n"
 	"\n"
-	"The following options are useful for people managing distributions;\n"
-	"Option '-b basedirectory': use an image of a module tree.\n"
-	"Option '-C configfile': use the file instead of /etc/modules.conf.\n"
-	"Option '-F kernelsyms': use the file instead of the current kernel symbols.\n"
+	"Options:\n"
+	"\t-a, --all            Probe modules listed in " ETC_MODULES_CONF "\n"
+	"\t-A                   Like -a, compares timestamps first\n"
+	"\t-q, --quiet          Don't report missing symbols\n"
+	"\t-e, --errsyms        List unresolved symbols for the given module\n"
+	"\t-s, --syslog         Report errors using syslog\n"
+	"\t-v, --verbose        Print all visited modules\n"
+	"\t-n, --show           Write the dependency file on stdout only\n"
+	"\t-r, --root           Allow root to allow modules not owned by root\n"
+	"\t-V, --version        Print the release version\n"
+	"\t-h, --help           Print this usage message\n"
+	"\n"
+	"The following options are useful for people managing distributions:\n"
+	"\t-b basedirectory\n"
+	"\t    --basedir basedirectory    Use an image of a module tree.\n"
+	"\t-C configfile\n"
+	"\t    --config configfile        Use the file instead of\n"
+	"\t                               " ETC_MODULES_CONF ".\n"
+	"\t-F kernelsyms\n"
+	"\t    --filesyms kernelsyms      Use the file instead of the\n"
+	"\t                               current kernel symbols.\n"
 	);
 }
 #endif  /* defined(COMMON_3264) && defined(ONLY_32) */
@@ -601,10 +660,10 @@ void depmod_usage(void)
 #if defined(COMMON_3264) && defined(ONLY_64)
 int depmod_main(int argc, char **argv)
 {
-        if (arch64())
-                return depmod_main_64(argc, argv);
-        else
-                return depmod_main_32(argc, argv);
+	if (arch64())
+		return depmod_main_64(argc, argv);
+	else
+		return depmod_main_32(argc, argv);
 }
 #endif  /* defined(COMMON_3264) && defined(ONLY_64) */
 
@@ -618,13 +677,14 @@ int DEPMOD_MAIN(int argc, char **argv)
 	char *conf_file = NULL;
 	char *file_syms = NULL;
 	char *base_dir = NULL;
+	int ignore_suffix = 0;	/* Ignore genksyms suffix on resolve? */
 
 	struct option long_opts[] = {
 		{"all", 0, 0, 'a'},
 		{"basedir", 1, 0, 'b'},
 		{"config", 1, 0, 'C'},
 		{"errsyms", 0, 0, 'e'},
-		{"filesysm", 1, 0, 'F'},
+		{"filesyms", 1, 0, 'F'},
 		{"help", 0, 0, 'h'},
 		{"show", 0, 0, 'n'},
 		{"quiet", 0, 0, 'q'},
@@ -706,6 +766,8 @@ int DEPMOD_MAIN(int argc, char **argv)
 	argc -= optind;
 	argv += optind;
 
+	ignore_suffix = file_syms != NULL;
+
 	if (stdmode || argc == 0) {
 		/* option -a is the default without arguments */
 		if (argc > 0) {
@@ -721,19 +783,15 @@ int DEPMOD_MAIN(int argc, char **argv)
 			int i;
 			GLOB_LIST *g = config_lstmod(NULL, NULL, 0);
 			for (i = 0; g && i < g->pathc; i++)
-				loadobj(g->pathv[i]);
+				loadobj(g->pathv[i], ignore_suffix);
 			resolve();
-
-			if (nflag)
-				prtdepend(NULL, base_dir);
-			else
-				prtdepend(depfile, base_dir);
+			prtdepend(nflag ? NULL : depfile, base_dir);
 		}
 	} else {
 		/* not stdmode */
 		if ((ret = addksyms(file_syms)) != -1) {
 			for (; argc > 0 && ret != -1; ++argv, --argc)
-				loadobj(*argv);
+				loadobj(*argv, ignore_suffix);
 			resolve();
 			prtdepend(NULL, base_dir);
 		}
