@@ -24,7 +24,7 @@
 #ifndef MODUTILS_OBJ_H
 #define MODUTILS_OBJ_H 1
 
-#ident "$Id: obj.h 1.2 Mon, 22 Nov 1999 01:38:27 +1100 keith $"
+#ident "$Id: obj.h 1.4 Tue, 18 Apr 2000 13:38:59 +1000 kaos $"
 
 /* The relocatable object is manipulated using elfin types.  */
 
@@ -148,7 +148,7 @@ struct obj_symbol_patch_struct
 #define obj_extend_section		ObjW(extend_section)
 #define obj_string_patch		ObjW(string_patch)
 #define obj_symbol_patch		ObjW(symbol_patch)
-#define obj_check_undefineds 		ObjW(check_undefineds)
+#define obj_check_undefineds		ObjW(check_undefineds)
 #define obj_allocate_commons		ObjW(allocate_commons)
 #define obj_load_size			ObjW(load_size)
 #define obj_relocate			ObjW(relocate)
@@ -163,9 +163,9 @@ struct obj_symbol_patch_struct
 #define arch_init_module		ObjW(arch_init_module)
 #define arch_load_proc_section		ObjW(arch_load_proc_section)
 
-unsigned long obj_elf_hash(const char *);
+unsigned long obj_elf_hash (const char *);
 
-unsigned long obj_elf_hash_n(const char *, unsigned long len);
+unsigned long obj_elf_hash_n (const char *, unsigned long len);
 
 struct obj_symbol *obj_add_symbol (struct obj_file *f, const char *name,
 				   unsigned long symidx, int info, int secidx,
@@ -174,10 +174,10 @@ struct obj_symbol *obj_add_symbol (struct obj_file *f, const char *name,
 struct obj_symbol *obj_find_symbol (struct obj_file *f,
 					 const char *name);
 
-ElfW(Addr) obj_symbol_final_value(struct obj_file *f,
+ElfW(Addr) obj_symbol_final_value (struct obj_file *f,
 				  struct obj_symbol *sym);
 
-void obj_set_symbol_compare(struct obj_file *f,
+void obj_set_symbol_compare (struct obj_file *f,
 			    int (*cmp)(const char *, const char *),
 			    unsigned long (*hash)(const char *));
 
@@ -199,26 +199,28 @@ struct obj_section *obj_create_alloced_section_first (struct obj_file *f,
 
 void *obj_extend_section (struct obj_section *sec, unsigned long more);
 
-int obj_string_patch(struct obj_file *f, int secidx, ElfW(Addr) offset,
+int obj_string_patch (struct obj_file *f, int secidx, ElfW(Addr) offset,
 		     const char *string);
 
-int obj_symbol_patch(struct obj_file *f, int secidx, ElfW(Addr) offset,
+int obj_symbol_patch (struct obj_file *f, int secidx, ElfW(Addr) offset,
 		     struct obj_symbol *sym);
 
-int obj_check_undefineds(struct obj_file *f, int quiet);
+int obj_check_undefineds (struct obj_file *f, int quiet);
 
-void obj_allocate_commons(struct obj_file *f);
+void obj_allocate_commons (struct obj_file *f);
 
 unsigned long obj_load_size (struct obj_file *f);
 
 int obj_relocate (struct obj_file *f, ElfW(Addr) base);
 
-struct obj_file *obj_load(FILE *f);
+struct obj_file *obj_load (FILE *f, Elf32_Half e_type);
 
-void obj_free(struct obj_file *f);
+void obj_free (struct obj_file *f);
 
 int obj_create_image (struct obj_file *f, char *image);
 
+int obj_kallsyms (struct obj_file *fin, struct obj_file **fout);
+
 /* Architecture specific manipulation routines.  */
 
 struct obj_file *arch_new_file (void);
@@ -238,6 +240,6 @@ int arch_create_got (struct obj_file *f);
 struct module;
 int arch_init_module (struct obj_file *f, struct module *);
 
-int arch_load_proc_section(struct obj_section *sec, FILE *fp);
+int arch_load_proc_section (struct obj_section *sec, FILE *fp);
 
 #endif /* obj.h */
