@@ -4,7 +4,7 @@
    New implementation contributed by Richard Henderson <rth@tamu.edu>
    Based on original work by Bjorn Ekwall <bj0rn@blox.se>
    Restructured (and partly rewritten) by:
-   	Bjrn Ekwall <bj0rn@blox.se> February 1999
+	Bjrn Ekwall <bj0rn@blox.se> February 1999
 
    This file is part of the Linux modutils.
 
@@ -39,14 +39,16 @@
      More flexible recognition of the way the utility was called.
      Suggested by Stepan Kasal, implemented in a different way by Keith
      Owens <kaos@ocs.com.au> December 1999.
- 
+
      Rationalize common code for 32/64 bit architectures.
        Keith Owens <kaos@ocs.com.au> December 1999.
      Add arch64().
        Keith Owens <kaos@ocs.com.au> December 1999.
+     kallsyms support
+       Keith Owens <kaos@ocs.com.au> April 2000.
    */
 
-#ident "$Id: insmod.c 1.12 Wed, 15 Mar 2000 23:50:08 +1100 keith $"
+#ident "$Id: insmod.c 1.17 Fri, 21 Apr 2000 13:50:20 +1000 kaos $"
 
 #include <sys/types.h>
 #include <stdlib.h>
@@ -63,6 +65,7 @@
 
 #include "module.h"
 #include "obj.h"
+#include "kallsyms.h"
 #include "util.h"
 #include "version.h"
 
@@ -90,6 +93,7 @@ extern int modprobe_main(int argc, char **argv);
 extern int rmmod_main(int argc, char **argv);
 extern int ksyms_main(int argc, char **argv);
 extern int lsmod_main(int argc, char **argv);
+extern int kallsyms_main(int argc, char **argv);
 
 /*======================================================================*/
 
@@ -116,7 +120,7 @@ static int get_kernel_version(char str[STRVERSIONLEN])
 	return a << 16 | b << 8 | c;
 }
 
-/* String comparison for non-co-versioned kernel and module. 
+/* String comparison for non-co-versioned kernel and module.
  * prefix should be the same as used by genksyms for this kernel.
  */
 static char *ncv_prefix = NULL;	/* Overridden by --prefix option */
@@ -183,7 +187,7 @@ static int ncv_strcmp(const char *a, const char *b)
 	    !(ncv_plen && strncmp(b + alen + 2, ncv_prefix, ncv_plen))) {
 		return strncmp(a, b, alen);
 	} else if (alen == blen + 10 + ncv_plen &&
-	           a[blen] == '_' && a[blen + 1] == 'R' &&
+		   a[blen] == '_' && a[blen + 1] == 'R' &&
 		   !(ncv_plen && strncmp(a + blen + 2, ncv_prefix, ncv_plen))) {
 		return strncmp(a, b, blen);
 	} else
@@ -599,10 +603,12 @@ static int is_module_checksummed(struct obj_file *f)
 		return obj_find_symbol(f, "Using_Versions") != NULL;
 }
 
-/* add module source, timestamp, kernel version and a symbol for the start of some sections.
- * this info is used by ksymoops to do better debugging.
+/* add module source, timestamp, kernel version and a symbol for the
+ * start of some sections.  this info is used by ksymoops to do better
+ * debugging.
  */
-static void add_ksymoops_symbols(struct obj_file *f, const char *filename, const char *m_name)
+static void add_ksymoops_symbols(struct obj_file *f, const char *filename,
+				 const char *m_name)
 {
 	struct obj_section *sec;
 	struct obj_symbol *sym;
@@ -921,7 +927,63 @@ static int process_module_arguments(struct obj_file *f, int argc, char **argv)
 	return 1;
 }
 
-static int init_module(const char *m_name, struct obj_file *f, unsigned long m_size)
+
+/* Add a kallsyms section if the kernel supports all symbols. */
+static int add_kallsyms(struct obj_file *f,
+			struct obj_section **module_kallsyms)
+{
+	struct module_symbol *s;
+	struct obj_file *f_kallsyms;
+	struct obj_section *sec_kallsyms;
+	size_t i;
+	int l;
+	const char *p, *pt_R;
+	unsigned long start = 0, stop = 0;
+
+	for (i = 0, s = ksyms; i < nksyms; ++i, ++s) {
+		p = (char *)s->name;
+		pt_R = strstr(p, "_R");
+		if (pt_R)
+			l = pt_R - p;
+		else
+			l = strlen(p);
+		if (strncmp(p, "__start_" KALLSYMS_SEC_NAME, l) == 0)
+			start = s->value;
+		else if (strncmp(p, "__stop_" KALLSYMS_SEC_NAME, l) == 0)
+			stop = s->value;
+	}
+
+	if (start >= stop)
+		return(0);
+
+	/* The kernel contains all symbols, do the same for this module. */
+
+	/* Add an empty kallsyms section to the module if necessary */
+	for (i = 0; i < f->header.e_shnum; ++i) {
+		if (strcmp(f->sections[i]->name, KALLSYMS_SEC_NAME) == 0) {
+			*module_kallsyms = f->sections[i];
+			break;
+		}
+	    }
+	if (!*module_kallsyms)
+		*module_kallsyms = obj_create_alloced_section(f, KALLSYMS_SEC_NAME, 0, 0);
+
+	/* Size and populate kallsyms */
+	if (obj_kallsyms(f, &f_kallsyms))
+		return(1);
+	sec_kallsyms = f_kallsyms->sections[KALLSYMS_IDX];
+	(*module_kallsyms)->header.sh_addralign = sec_kallsyms->header.sh_addralign;
+	(*module_kallsyms)->header.sh_size = sec_kallsyms->header.sh_size;
+	free((*module_kallsyms)->contents);
+	(*module_kallsyms)->contents = sec_kallsyms->contents;
+	sec_kallsyms->contents = NULL;
+	obj_free(f_kallsyms);
+
+	return 0;
+}
+
+static int init_module(const char *m_name, struct obj_file *f,
+		       unsigned long m_size)
 {
 	struct module *module;
 	struct obj_section *sec;
@@ -948,7 +1010,8 @@ static int init_module(const char *m_name, struct obj_file *f, unsigned long m_s
 		module->ndeps = n_ext_modules_used;
 	}
 	module->init = obj_symbol_final_value(f, obj_find_symbol(f, "init_module"));
-	module->cleanup = obj_symbol_final_value(f, obj_find_symbol(f, "cleanup_module"));
+	module->cleanup = obj_symbol_final_value(f,
+		obj_find_symbol(f, "cleanup_module"));
 
 	sec = obj_find_section(f, "__ex_table");
 	if (sec) {
@@ -965,6 +1028,11 @@ static int init_module(const char *m_name, struct obj_file *f, unsigned long m_s
 		    module->runsize > sec->header.sh_addr - m_addr)
 			module->runsize = sec->header.sh_addr - m_addr;
 	}
+	sec = obj_find_section(f, KALLSYMS_SEC_NAME);
+	if (sec && sec->header.sh_size) {
+		module->kallsyms_start = sec->header.sh_addr;
+		module->kallsyms_end = module->kallsyms_start + sec->header.sh_size;
+	}
 	if (!arch_init_module(f, module))
 		return 0;
 
@@ -978,9 +1046,9 @@ static int init_module(const char *m_name, struct obj_file *f, unsigned long m_s
 	ret = sys_init_module(m_name, (struct module *) image);
 	if (ret) {
 		error("init_module: %m");
-		if (ret == -EBUSY)
+		if (errno == EBUSY)
 			lprintf("Hint: this error can be caused by incorrect module parameters, "
-				"including invalid IO and IRQ parameters\n");
+				"including invalid IO or IRQ parameters");
 	}
 
 	free(image);
@@ -989,7 +1057,8 @@ static int init_module(const char *m_name, struct obj_file *f, unsigned long m_s
 }
 
 #ifdef COMPAT_2_0
-static int old_init_module(const char *m_name, struct obj_file *f, unsigned long m_size)
+static int old_init_module(const char *m_name, struct obj_file *f,
+			   unsigned long m_size)
 {
 	char *image;
 	struct old_mod_routines routines;
@@ -1014,7 +1083,7 @@ static int old_init_module(const char *m_name, struct obj_file *f, unsigned long
 	}
 	total = (sizeof(struct old_symbol_table) +
 		 nsyms * sizeof(struct old_module_symbol) +
-	         n_ext_modules_used * sizeof(struct old_module_ref) +
+		 n_ext_modules_used * sizeof(struct old_module_ref) +
 		 strsize);
 	symtab = xmalloc(total);
 	symtab->size = total;
@@ -1061,7 +1130,8 @@ static int old_init_module(const char *m_name, struct obj_file *f, unsigned long
 	/* Fill in routines.  */
 
 	routines.init = obj_symbol_final_value(f, obj_find_symbol(f, "init_module"));
-	routines.cleanup = obj_symbol_final_value(f, obj_find_symbol(f, "cleanup_module"));
+	routines.cleanup = obj_symbol_final_value(f,
+		obj_find_symbol(f, "cleanup_module"));
 
 	/*
 	 * Whew!  All of the initialization is complete.
@@ -1078,7 +1148,7 @@ static int old_init_module(const char *m_name, struct obj_file *f, unsigned long
 	 */
 	ret = old_sys_init_module(m_name, image + sizeof(long),
 				  (m_size - sizeof(long)) |
-			          (flag_autoclean ? OLD_MOD_AUTOCLEAN : 0),
+				  (flag_autoclean ? OLD_MOD_AUTOCLEAN : 0),
 				  &routines,
 				  symtab);
 	if (ret)
@@ -1101,25 +1171,27 @@ extern void insmod_usage(void);		/* Use the copy in the 64 bit version */
 void insmod_usage(void)
 {
 	fputs("Usage:\n"
-	      "insmod [-fkmopsvVxXyY] [-o name] [-P prefix] module [[sym=value]...]\n"
+	      "insmod [-fhkmopqsvVxXyY] [-o name] [-P prefix] module [[sym=value]...]\n"
 
 	      "\n"
 	      "  module                Filename of a loadable kernel module (*.o)\n"
 	      "  -f, --force           Force loading under wrong kernel version\n"
+	      "  -h, --help            Print this message.\n"
 	      "  -k, --autoclean       Make module autoclean-able\n"
 	      "  -m, --map             Generate load map (so crashes can be traced)\n"
 	      "  -n, --noload          Don't load, just show\n"
 	      "  -o NAME, --name=NAME  Set internal module name to NAME\n"
 	      "  -p, --poll            Poll mode; check if the module matches the kernel\n"
+	      "  -q, --quiet           Don't print unresolved symbols\n"
 	      "  -s, --syslog          Report errors via syslog\n"
 	      "  -v, --verbose         Verbose output\n"
 	      "  -L, --lock            Prevent simultaneous loads of the same module\n"
 	      "  -V, --version         Show version\n"
-	      "  -x                    Do not export externs\n"
-	      "  -X                    Do export externs (default)\n"
-	      "  -y                    Do not add ksymoops symbols\n"
-	      "  -Y                    Do add ksymoops symbols (default)\n"
-	      "  -r                    Allow root to load modules not owned by root\n"
+	      "  -x, --noexport        Do not export externs\n"
+	      "  -X, --export          Do export externs (default)\n"
+	      "  -y, --noksymoops      Do not add ksymoops symbols\n"
+	      "  -Y, --ksymoops        Do add ksymoops symbols (default)\n"
+	      "  -r, --root            Allow root to load modules not owned by root\n"
 	      "  -P PREFIX\n"
 	      "      --prefix=PREFIX   Prefix for kernel or module symbols\n"
 	      ,stderr);
@@ -1158,6 +1230,7 @@ int INSMOD_MAIN(int argc, char **argv)
 		{"noksymoops", 0, 0, 'y'},
 		{"ksymoops", 0, 0, 'Y'},
 		{"root", 0, 0, 'r'},
+		{"help", 0, 0, 'h'},
 		{0, 0, 0, 0}
 	};
 	char *m_name = NULL;
@@ -1169,6 +1242,7 @@ int INSMOD_MAIN(int argc, char **argv)
 	char *filename;
 	FILE *fp;
 	struct obj_file *f;
+	struct obj_section *kallsyms = NULL;
 	int o;
 	int noload = 0;
 	int dolock = 1; /*Note: was: 0; */
@@ -1182,7 +1256,7 @@ int INSMOD_MAIN(int argc, char **argv)
 	errors = optind = 0;
 
 	/* Process the command line.  */
-	while ((o = getopt_long(argc, argv, "fkmno:pqsvVxXLP:yYr",
+	while ((o = getopt_long(argc, argv, "fhkmno:pqsvVxXLP:yYr",
 				&long_opts[0], NULL)) != EOF)
 		switch (o) {
 		case 'f':	/* force loading */
@@ -1236,6 +1310,7 @@ int INSMOD_MAIN(int argc, char **argv)
 		case 'P':	/* use prefix on crc */
 			set_ncv_prefix(optarg);
 			break;
+		case 'h':       /* Print the usage message. */
 		default:
 			insmod_usage();
 			break;
@@ -1309,7 +1384,7 @@ int INSMOD_MAIN(int argc, char **argv)
 	}
 
 	error_file = filename;
-	if ((f = obj_load(fp)) == NULL)
+	if ((f = obj_load(fp, ET_REL)) == NULL)
 		goto out;
 
 	/* Version correspondence?  */
@@ -1371,6 +1446,11 @@ int INSMOD_MAIN(int argc, char **argv)
 	if (k_new_syscalls)
 		create_module_ksymtab(f);
 
+	/* kallsyms based on relocatable addresses */
+	if (add_kallsyms(f, &kallsyms))
+		goto out;
+	/**** No symbols or sections to be changed after kallsyms above ***/
+
 	if (errors)
 		goto out;
 
@@ -1418,6 +1498,11 @@ int INSMOD_MAIN(int argc, char **argv)
 			delete_module(m_name);
 		goto out;
 	}
+
+	/* Do kallsyms again, this time we have the final addresses */
+	if (add_kallsyms(f, &kallsyms))
+		goto out;
+
 	if (!noload) {
 #ifdef COMPAT_2_0
 		if (k_new_syscalls)
@@ -1484,6 +1569,9 @@ int main(int argc, char **argv)
 	#endif
 	#ifdef COMBINE_lsmod
 			{ "lsmod", &lsmod_main },
+	#endif
+	#ifdef COMBINE_kallsyms
+			{ "kallsyms", &kallsyms_main },
 	#endif
 		};
 	#define MAINS_NO (sizeof(mains)/sizeof(mains[0]))
@@ -1521,7 +1609,8 @@ int main(int argc, char **argv)
 	if (mains_match == 0 && MAINS_NO == 1)
 		++mains_match;		/* Not combined, any name will do */
 	if (mains_match == 0) {
-		error("%s does not have a recognisable name, the name must contain one of %s.",
+		error("%s does not have a recognisable name, "
+		      "the name must contain one of %s.",
 			error_id1, error_id2);
 		return(1);
 	}
