@@ -21,9 +21,10 @@
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ident "$Id: ksyms.c 1.1 Wed, 25 Aug 1999 16:26:49 +1000 keith $"
+#ident "$Id: ksyms.c 1.2 Thu, 13 Apr 2000 18:17:59 +1000 kaos $"
 
 #include <unistd.h>
+#include <getopt.h>
 
 #include "module.h"
 #include "version.h"
@@ -35,6 +36,19 @@
 #define main ksyms_main
 #endif
 
+void ksyms_usage(void)
+{
+	fputs("Usage:\n"
+	      "ksyms [-ahm?V]\n"
+	      "\n"
+	      " -a, --all      Display all symbols\n"
+	      " -h, --noheader Suppress the column header\n"
+	      " -m, --info     Display module information\n"
+	      " -H, --help     Print this message\n"
+	      " -V, --version  Print the release version\n"
+	      ,stderr);
+}
+
 /*FIXME: A non-root ksyms: copy the output from /proc/ksyms */
 int main(int argc, char **argv)
 {
@@ -46,9 +60,19 @@ int main(int argc, char **argv)
 	int flag_mod_info = 0;
 	int flag_print_header = 1;
 
+	struct option long_opts[] = {
+		{"all", 0, 0, 'a'},
+		{"info", 0, 0, 'm'},
+		{"noheader", 0, 0, 'h'},
+		{"version", 0, 0, 'V'},
+		{"help", 0, 0, 'H'},
+		{0, 0, 0, 0}
+	};
+
 	error_file = "ksyms";
 
-	while ((i = getopt(argc, argv, "amhV")) != EOF)
+	while ((i = getopt_long(argc, argv, "amhVH",
+				&long_opts[0], NULL)) != EOF)
 		switch (i) {
 		case 'a':
 			flag_allsyms = 1;
@@ -62,6 +86,12 @@ int main(int argc, char **argv)
 		case 'V':
 			fputs("ksyms version " MODUTILS_VERSION "\n", stderr);
 			break;
+		case 'H':
+			ksyms_usage();
+			return 0;
+		default:
+			ksyms_usage();
+			return 1;
 		}
 
 	if (!get_kernel_info(K_SYMBOLS | K_INFO))
