@@ -21,31 +21,63 @@
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ident "$Id: lsmod.c 1.2 Fri, 26 Nov 1999 00:00:09 +1100 keith $"
+#ident "$Id: lsmod.c 1.3 Thu, 13 Apr 2000 18:17:59 +1000 kaos $"
 
 #include <stdio.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <getopt.h>
 
 #include "module.h"
 #include "util.h"
 #include "obj.h"
 #include "modstat.h"
+#include "version.h"
 
 #ifdef COMBINE_lsmod
 #define main lsmod_main
 #endif
 
+void lsmod_usage(void)
+{
+	fputs("Usage:\n"
+	      "lsmod [-Vh]\n"
+	      "\n"
+	      " -V, --version    Print the release version\n"
+	      " -h, --help       Print this message\n"
+	      ,stderr);
+}
+
 int main(int argc, char **argv)
 {
 	struct module_stat *m;
 	int i;
 	int j;
 
+	struct option long_opts[] = {
+	  {"version", 0, 0, 'V'},
+	  {"help", 0, 0, 'h'},
+	  {0, 0, 0, 0}
+	};
+
 	error_file = "lsmod";
 
+	while ((i = getopt_long(argc, argv, "Vh",
+				&long_opts[0], NULL)) != EOF)
+		switch(i) {
+		case 'V':
+			fputs("lsmod version " MODUTILS_VERSION "\n",stderr);
+			return 0;
+		case 'h':
+			lsmod_usage();
+			return 0;
+		default:
+			lsmod_usage();
+			return 1;
+		}
+
 	/* A header.  */
 	puts("Module                  Size  Used by");
 
