@@ -31,7 +31,7 @@
  *   Keith Owens <kaos@ocs.com.au> December 1999.
  */
 
-#ident "$Id: modinfo.c 1.6 Sun, 26 Dec 1999 20:47:58 +1100 keith $"
+#ident "$Id: modinfo.c 1.8 Thu, 20 Apr 2000 14:31:49 +1000 kaos $"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -134,11 +134,13 @@ static char *format_query_string(struct obj_file *f, char *format)
 			break;
 
 		case 'a':
-			(void) append_modinfo_tag(f, "author", "<none>", &out_str, last_char - out_str);
+			(void) append_modinfo_tag(f, "author", "<none>",
+				&out_str, last_char - out_str);
 			break;
 
 		case 'd':
-			(void) append_modinfo_tag(f, "description", "<none>", &out_str, last_char - out_str);
+			(void) append_modinfo_tag(f, "description", "<none>",
+				&out_str, last_char - out_str);
 			break;
 
 		case '{':{
@@ -178,7 +180,8 @@ static char *format_query_string(struct obj_file *f, char *format)
 	return &buffer[0];
 }
 
-static void show_parameter(struct obj_file *f, char *key, char *value, char *desc)
+static void show_parameter(struct obj_file *f, char *key, char *value,
+			   char *desc)
 {
 	struct obj_symbol *sym;
 	int min, max;
@@ -303,7 +306,7 @@ int SHOW_MODULE_INFO(char *filename, char *fmtstr, int do_parameters)
 	if ((fp = fopen(filename, "r")) == NULL) {
 		error("%s: %m", filename);
 		return -1;
-	} else if ((f = obj_load(fp)) == NULL)
+	} else if ((f = obj_load(fp, ET_REL)) == NULL)
 		return -1;
 	fclose(fp);
 
