@@ -368,7 +368,8 @@ static DESC *build_desc(const char *name, const char *objname, int allow_alias)
  *	aliased off	-1
  *	error		-2
  */
-static int build_stack(LINK **stack, const char *name, const char *objname, int dir, DESC *rev, int mode, int allow_alias)
+static int build_stack(LINK **stack, const char *name, const char *objname,
+		       int dir, DESC *rev, int mode, int allow_alias)
 {
 	NODE *nod;
 	DESC *desc;
@@ -458,7 +459,8 @@ static int build_stack(LINK **stack, const char *name, const char *objname, int
 	 * not allowed.
 	 */
 	for (nod = desc->nod->dep; nod; nod = nod->next) {
-		if ((r = build_stack(stack, nod->key, nod->str, dir, rev, MODE_NORMAL, 0)) < 0)
+		if ((r = build_stack(stack, nod->key, nod->str, dir, rev,
+			MODE_NORMAL, 0)) < 0)
 			return r;
 	}
 
@@ -1236,7 +1238,8 @@ static int unload(const char *name)
  *	Install OK:	0	asked_for points to the requested module node
  *
  */
-static int probe_stack(const char *name, const char *obj, char **options, DESC **asked_for)
+static int probe_stack(const char *name, const char *obj, char **options,
+		       DESC **asked_for)
 {
 	LINK *stack;
 	LINK *p;
@@ -1309,7 +1312,7 @@ static int modprobe_fromlist(int argc, char *argv[], char *type, int loadall)
 		 * #
 		 * An option is a keyword followed by an equal sign and a value.
 		 * No spaces are allowed in the sequence, unless quoted.
-		 * 
+		 *
 		 * The option list ends at the end of the list or at the
 		 * first non-option argument (a possible next module).
 		 */
@@ -1417,9 +1420,12 @@ static void usage(void)
 		"options:\n"
 		"\t-a, --all                  Load _all_ matching modules\n"
 		"\t-c, --showconfig           Show current configuration\n"
+		"\t-d, --debug                Print debugging information\n"
+		"\t-h, --help                 Print this message\n"
 		"\t-k, --autoclean            Set 'autoclean' on loaded modules\n"
 		"\t-l, --list                 List matching modules\n"
 		"\t-n, --show                 Don't actually perform the action\n"
+		"\t-q, --quiet                Quiet operation\n"
 		"\t-r, --remove               Remove module (stacks) or do autoclean\n"
 		"\t-s, --syslog               Use syslog to report\n"
 		"\t-t, --type moduletype      Only look for modules of this type\n"
@@ -1452,12 +1458,13 @@ int main(int argc, char *argv[])
 		{"syslog", 0, 0, 's'},
 		{"config", 1, 0, 'C'},
 		{"quiet", 1, 0, 'q'},
+		{"help", 0, 0, 'h'},
 		{0, 0, 0, 0}
 	};
 
 	error_file = "modprobe";
 
-	while ((o = getopt_long(argc, argv, "t:acdlnqrvksC:V",
+	while ((o = getopt_long(argc, argv, "t:acdhlnqrvksC:V",
 				&long_opts[0], NULL)) != EOF) {
 		switch (o) {
 		case 't': /* type of module */
@@ -1475,7 +1482,10 @@ int main(int argc, char *argv[])
 		case 'd':
 			debug = 1;
 			break;
-
+		case 'h':
+			usage();
+			return 0;
+			break;
 		case 'l':
 			list = 1;
 			break;
