@@ -3,6 +3,7 @@
 
    Contributed by Richard Henderson <rth@tamu.edu>
    obj_free() added by Bjrn Ekwall <bj0rn@blox.se> March 1999
+   Support for kallsyms Keith Owens <kaos@ocs.com.au> April 2000
 
    This file is part of the Linux modutils.
 
@@ -20,7 +21,7 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-#ident "$Id: obj_load.c 1.1 Wed, 25 Aug 1999 16:26:49 +1000 keith $"
+#ident "$Id: obj_load.c 1.4 Thu, 20 Apr 2000 14:31:49 +1000 kaos $"
 
 #include <alloca.h>
 #include <string.h>
@@ -32,7 +33,7 @@
 /*======================================================================*/
 
 struct obj_file *
-obj_load (FILE *fp)
+obj_load (FILE *fp, Elf32_Half e_type)
 {
   struct obj_file *f;
   ElfW(Shdr) *section_headers;
@@ -70,9 +71,20 @@ obj_load (FILE *fp)
       error("ELF file not for this architecture");
       return NULL;
     }
-  if (f->header.e_type != ET_REL)
+  if (f->header.e_type != e_type && e_type != ET_NONE)
     {
-      error("ELF file not a relocatable object");
+      switch (e_type) {
+      case ET_REL:
+	error("ELF file not a relocatable object");
+	break;
+      case ET_EXEC:
+	error("ELF file not an executable object");
+	break;
+      default:
+	error("ELF file has wrong type, expecting %d got %d",
+		e_type, f->header.e_type);
+	break;
+      }
       return NULL;
     }
 
@@ -111,7 +123,7 @@ obj_load (FILE *fp)
       sec->idx = i;
 
       switch (sec->header.sh_type)
- 	{
+	{
 	case SHT_NULL:
 	case SHT_NOTE:
 	case SHT_NOBITS:
@@ -127,10 +139,10 @@ obj_load (FILE *fp)
 	      sec->contents = xmalloc(sec->header.sh_size);
 	      fseek(fp, sec->header.sh_offset, SEEK_SET);
 	      if (fread(sec->contents, sec->header.sh_size, 1, fp) != 1)
-	        {
-	          error("error reading ELF section data: %m");
-	          return NULL;
-	        }
+		{
+		  error("error reading ELF section data: %m");
+		  return NULL;
+		}
 	    }
 	  else
 	    sec->contents = NULL;
@@ -150,7 +162,7 @@ obj_load (FILE *fp)
 	  if (sec->header.sh_type >= SHT_LOPROC)
 	    {
 	      if (arch_load_proc_section(sec, fp) < 0)
-	        return NULL;
+		return NULL;
 	      break;
 	    }
 
