@@ -269,7 +269,8 @@ static GLOB_LIST *addlist(GLOB_LIST *orig, GLOB_LIST *add)
 	return orig;
 }
 
-static void decode_list(int *n, OPT_LIST **list, char *arg, int adding, char *version)
+static void decode_list(int *n, OPT_LIST **list, char *arg, int adding,
+			char *version)
 {
 	GLOB_LIST *pg;
 	GLOB_LIST *prevlist = NULL;
@@ -641,8 +642,8 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file)
 
 	if (fin) {
 		struct stat statbuf1, statbuf2;
-		if (fstat(fileno(fin), &statbuf1) == 0) 
-			config_mtime = statbuf1.st_mtime; 
+		if (fstat(fileno(fin), &statbuf1) == 0)
+			config_mtime = statbuf1.st_mtime;
 		config_file = xstrdup(conf_file);	/* Save name actually used */
 		if (!conf_file_specified &&
 		    stat(ETC_MODULES_CONF, &statbuf1) == 0 &&
@@ -655,7 +656,7 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file)
 					fprintf(stderr,
 						"Warning: You do not need a link from %s to\n"
 						"         %s.  The use of %s is depreciated,\n"
-						"         please remove %s and rename %s\n" 
+						"         please remove %s and rename %s\n"
 						"         to %s as soon as possible.  Commands.\n"
 						"           rm %s\n"
 						"           mv %s %s\n",
@@ -806,8 +807,8 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file)
 
 			if (strncmp(arg, "-f", 2) == 0) {
 				char *file = next_word(arg);
-
-				if (access(file, R_OK) == 0)
+				meta_expand(file, &g, NULL, version);
+				if (access(g.pathc ? g.pathv[0] : file, R_OK) == 0)
 					state[level] = !not;
 				else
 					state[level] = not;
@@ -845,32 +846,32 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file)
 
 				if (strcmp(cmp, "==") == 0 ||
 				    strcmp(cmp, "=") == 0) {
-				    	if (numeric)
+					if (numeric)
 					    state[level] = (n1 == n2);
 					else
 					    state[level] = strcmp(w1, w2) == 0;
 				} else if (strcmp(cmp, "!=") == 0) {
-				    	if (numeric)
+					if (numeric)
 					    state[level] = (n1 != n2);
 					else
 					    state[level] = strcmp(w1, w2) != 0;
 				} else if (strcmp(cmp, ">=") == 0) {
-				    	if (numeric)
+					if (numeric)
 					    state[level] = (n1 >= n2);
 					else
 					    state[level] = strcmp(w1, w2) >= 0;
 				} else if (strcmp(cmp, "<=") == 0) {
-				    	if (numeric)
+					if (numeric)
 					    state[level] = (n1 <= n2);
 					else
 					    state[level] = strcmp(w1, w2) <= 0;
 				} else if (strcmp(cmp, ">") == 0) {
-				    	if (numeric)
+					if (numeric)
 					    state[level] = (n1 > n2);
 					else
 					    state[level] = strcmp(w1, w2) > 0;
 				} else if (strcmp(cmp, "<") == 0) {
-				    	if (numeric)
+					if (numeric)
 					    state[level] = (n1 < n2);
 					else
 					    state[level] = strcmp(w1, w2) < 0;
@@ -916,14 +917,12 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file)
 		 * include
 		 */
 		if (!assgn && strcmp(parm, "include") == 0) {
-			char incfile[PATH_MAX];
+			meta_expand(arg, &g, NULL, version);
 
-			sscanf(arg, "%s", incfile);
-
-			if (!do_read(all, version, base_dir, incfile))
+			if (!do_read(all, version, base_dir, g.pathc ? g.pathv[0] : arg))
 				one_err = 0;
 			else
-				error("include %s failed\n", incfile);
+				error("include %s failed\n", arg);
 		}
 
 		/*
@@ -1284,7 +1283,7 @@ static int config_add(const char *file, const struct stat *sb, int flag)
 			if ((p = strrchr(file, '/')) == NULL)
 				p = (char *)file;
 			else
-	    			p += 1;
+				p += 1;
 
 			if (strcmp(p, looking_for) != 0)
 				return 0;
