@@ -20,7 +20,7 @@
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
   */
 
-#ident "$Id: snap_shot.c 1.1 Wed, 25 Aug 1999 18:01:54 +1000 keith $"
+#ident "$Id: snap_shot.c 1.4 Mon, 22 Nov 1999 00:58:28 +1100 keith $"
 
 #include <errno.h>
 #include <stdio.h>
@@ -28,16 +28,18 @@
 #include <unistd.h>
 
 #include "module.h"
+#include "obj.h"
+#include "modstat.h"
 #include "util.h"
 
 /* If snap_dir exists, take a snap shot of ksyms and modules to snap_dir.
  * Prefix the files with the equivalent of
  * date +%Y%m%d%T%M%S | sed -e 's/://g'
  */
-void snap_shot(const char *module_names, int number)
+void snap_shot(const char *module_names, int n_module_names)
 {
 	static char snap_dir[] = "/var/log/ksymoops";
-	char wd[sizeof(snap_dir)+2], file[] = "ccyymmddhhmmss.modules", buffer[4096];
+	char file[] = "ccyymmddhhmmss.modules", buffer[4096];
 	static char *infile[] = { "/proc/ksyms", "/proc/modules" };
 	static char *suffix[] = {       "ksyms",       "modules" };
 	struct tm *local;
@@ -50,43 +52,33 @@ void snap_shot(const char *module_names, int number)
 		 * Otherwise auto cleanup takes a snap shot every time
 		 * and ends up with a large snap shot directory.
 		 */
-		size_t ret;
-		size_t bufsize;
 		char *new_module_names;
-		new_module_names = xmalloc(bufsize = 256);
-		while (query_module(NULL, QM_MODULES, new_module_names, bufsize, &ret)) {
-			if (errno != ENOSPC) {
-				error("QM_MODULES: %m\n");
-				return;
-			}
-			new_module_names = xrealloc(new_module_names, bufsize = ret);
-		}
-		if (number && ret == number) {
-			while (number) {
+		size_t n_new_module_names;
+		get_kernel_info(0);
+		new_module_names = module_name_list;
+		n_new_module_names = n_module_stat;
+		if (n_module_names && n_new_module_names == n_module_names) {
+			while (n_module_names) {
 				if (strcmp(module_names, new_module_names))
 					break;	/* difference detected */
 				i = strlen(module_names) + 1;
 				module_names += i;
 				new_module_names += i;
-				--number;
+				--n_module_names;
 			}
 		}
-		if (!number)
+		if (!n_module_names)
 			return;	/* no difference, no need for snap shot */
 	}
 
 	if (chdir(snap_dir))
 		return;
-	if (!getcwd(wd, sizeof(wd)) || strcmp(snap_dir, wd)) {
-		error("%s must be a real directory", snap_dir);
-		return;
-	}
 	t = time(NULL);
 	local = localtime(&t);
 	for (i = 0; i < sizeof(infile)/sizeof(infile[0]); ++i) {
 		snprintf(file, sizeof(file), "%04d%02d%02d%02d%02d%02d.%s",
 			local->tm_year+1900,
-			local->tm_mon,
+			local->tm_mon + 1,
 			local->tm_mday,
 			local->tm_hour,
 			local->tm_min,
